%% Author: Thomas Möllenhoff
function affMat = create_graph_zmp(X, NN, M, symType)
    % creates the affinity matrix using the procedure described in
    % "Self-tuning spectral clustering", Zelnik-Manor and Perona, NIPS 2010
    
    N = size(X, 1);
    %idx = knnsearch(X, X(i, :), 'K', max(NN, M));
    weights = sparse(N, N);
    for i=1:N
        %i
        idx = knnsearch(X, X(i, :), 'K', max(NN, M));
       
        % sq. distance between point i and its NN nearest neighbours
        num = sum((repmat(X(i,:), NN, 1) - X(idx(1:NN),:)).^2,2); 
    
        % distance between point i and the Mth nearest neighbour
        denom1 = sqrt(sum((X(i,:) - X(idx(M),:)).^2));
    
        % distance between the NN nearest neighbours of point i and the 
        % Mth nearest neighbour
        denom2 = sqrt(sum((X(idx(1:NN),:) - repmat(X(idx(M),:), NN, 1)).^2,2));
    
        % compute weights as in Eq.30
        weights(i, idx(1:NN)) = exp(-num./(denom1.*denom2+1e-6));
    end
    weights(logical(speye(size(weights)))) = 0; % set diagonal to zero

    %% symmetrize the affinity matrix
    if symType == 'max'
        affMat = max(weights, weights');
    elseif symType == 'min'
        affMat = min(weights, weights');
    elseif symType == 'avg'
        affMat = (weights + weights') / 2;
    elseif symType == 'bgr'
        bigger = weights' > weights;
        affMat = weights - weights .* bigger + weights' .* bigger;
    else
        affMat = weights;
    end
    
end
